-module (urlchecker).
-export ([start/0, fetch/2, main/0]).
-import (ibrowse).

start () ->
    ibrowse:start(),
    loop (sets:new(), []).

loop (Seen, Pids) ->
    receive
        {url, Url} ->
            case sets:is_element(Url, Seen) of
                true ->
                    io:fwrite ("~s seen previously.~n", [Url]),
                    loop(Seen, Pids);
                false ->
                    Pid = spawn(urlchecker, fetch, [self(), Url]),
                    loop(sets:add_element(Url, Seen), [Pid|Pids])
            end;
        {reply, Pid, Url, RC} ->
            io:fwrite ("~s : ~s~n", [Url, RC]),
            loop(Seen, lists:delete(Pid, Pids));
        done ->
            case Pids of
                [] ->
                    io:fwrite ("Exiting~n", []),
                    exit(done);
                _ ->
                    io:fwrite ("Waiting for children to finish~n", []),
                    timer:send_after (1000, done),
                    loop(Seen, Pids)
            end
    end.

fetch(Pid, Url) ->
    case ibrowse:send_req(Url, [], get, [], [], 30000) of
        {ok, Status, _, _} ->
            Pid ! {reply, self(), Url, Status};
        {error, Reason} ->
            io:fwrite ("Error fetching ~s : ~w~n", [Url, Reason])
    end.

main () ->
    Pid = spawn_link(urlchecker, start, []),
    main_loop(Pid).

main_loop (Pid) ->
    case io:get_line('') of
        eof ->
            Pid ! done;
        Line ->
            Pid ! {url, string:strip(Line, right, $\n)},
            main_loop(Pid)
    end.
