#!/usr/bin/env perl

use strict;
use warnings;

use POE qw(Component::Client::HTTP);
use HTTP::Request;

POE::Component::Client::HTTP->spawn(
  Alias       => 'client',
  Timeout     => 30,
);


sub get_next_url {
  my $next_url = <>;
  return undef unless defined $next_url;
  chomp $next_url;
  return $next_url;
}

POE::Session->create(
  inline_states => {
    _start => sub {
      my ($kernel, $heap) = @_[KERNEL,HEAP];

      $heap->{seen} = {};

      $kernel->yield("next_lookup");
    },

    _stop => sub {},

    next_lookup => sub {
      my ($kernel, $heap) = @_[KERNEL,HEAP];

      my $url = get_next_url();
      return unless defined $url;

      if ($heap->{seen}{$url}) {
        print("$url seen previously.\n");
      } else {
        $kernel->post('client',
                      'request',
                      'handle_response',
                      HTTP::Request->new(GET => $url));
        $heap->{seen}{$url} = 1;
      }

      $kernel->yield("next_lookup");
    },

    handle_response => sub {
      my ($req, $res) = @_[ARG0, ARG1];
      my $request = $req->[0];
      my $response = $res->[0];

      printf("%s : %s\n", $request->uri, $response->code);
    }
  }
);

$poe_kernel->run();
exit 0;

